//
// ********************************************************************
// * License and Disclaimer                                           *
// *                                                                  *
// * The  Geant4 software  is  copyright of the Copyright Holders  of *
// * the Geant4 Collaboration.  It is provided  under  the terms  and *
// * conditions of the Geant4 Software License,  included in the file *
// * LICENSE and available at  http://cern.ch/geant4/license .  These *
// * include a list of copyright holders.                             *
// *                                                                  *
// * Neither the authors of this software system, nor their employing *
// * institutes,nor the agencies providing financial support for this *
// * work  make  any representation or  warranty, express or implied, *
// * regarding  this  software system or assume any liability for its *
// * use.  Please see the license in the file  LICENSE  and URL above *
// * for the full disclaimer and the limitation of liability.         *
// *                                                                  *
// * This  code  implementation is the result of  the  scientific and *
// * technical work of the GEANT4 collaboration.                      *
// * By using,  copying,  modifying or  distributing the software (or *
// * any work based  on the software)  you  agree  to acknowledge its *
// * use  in  resulting  scientific  publications,  and indicate your *
// * acceptance of all terms of the Geant4 Software license.          *
// ********************************************************************
//
//
// $Id$
//
// 

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......
//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

#ifndef PhysicsList_h
#define PhysicsList_h 1

#include "G4VUserPhysicsList.hh"
#include "globals.hh"

#include "G4PiKBuilder_WP.hh"
#include "G4BertiniPiKBuilder.hh"
#include "G4FTFPPiKBuilder.hh"

#include "G4ProtonBuilder_WP.hh"
#include "G4BertiniProtonBuilder.hh"
#include "G4FTFPNeutronBuilder.hh"
#include "G4FTFPProtonBuilder.hh"

#include "G4NeutronBuilder_WP.hh"
#include "G4BertiniNeutronBuilder.hh"
#include "G4FTFPNeutronBuilder.hh"
// #include "G4LEPNeutronBuilder.hh"

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

class PhysicsList: public G4VUserPhysicsList
{
public:
  PhysicsList();
  virtual ~PhysicsList();

  // Construct particle and physics
  void ConstructParticle();
  void ConstructProcess();
 
  void SetCuts();
   
private:
   PhysicsList(const PhysicsList&); // Not implemented
   PhysicsList& operator=(const PhysicsList&); // Not implemented

  // these methods Construct physics processes and register them
  void ConstructDecay();
  void ConstructEM();
  // this is the wrapper process for a simplified HadronPhysicsTFTP_BERT
  void HadronPhysicsFTFP_BERT_WP();

private:
  G4NeutronBuilder_WP * theNeutrons;
  G4BertiniNeutronBuilder * theBertiniNeutron;
  G4FTFPNeutronBuilder * theFTFPNeutron;
  // G4LEPNeutronBuilder * theLEPNeutron;        //needed for capture&fission
 
  G4PiKBuilder_WP * thePiK;
  G4BertiniPiKBuilder * theBertiniPiK;
  G4FTFPPiKBuilder * theFTFPPiK;
    
  G4ProtonBuilder_WP * thePro;
  G4BertiniProtonBuilder * theBertiniPro;
  G4FTFPProtonBuilder * theFTFPPro;    

  G4bool QuasiElastic;
};

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

#endif



